import subprocess
import sys
import requests
from bs4 import BeautifulSoup
import pandas as pd

class TopMoviesScraper:
    def __init__(self):
        self.headers = {'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/58.0.3029.110 Safari/537.3'}
        self.user_viewers = []
        self.critic_viewers = []
        self.metadata = []
        self.rating = []
        self.viewers = []
        self.director = []
        self.writer = []
        self.release = []
        self.country_of_origin = []
        self.language = []
        self.also_known_as = []
        self.film_location = []
        self.title = []
        self.poster = []
        self.time = []
        self.movielink = []

    def install_requirements(self):
        try:
            subprocess.check_call([sys.executable, "-m", "pip", "install", "-U", "-r", "requirements.txt"])
        
        except subprocess.CalledProcessError:
            print("Failed to install 'requests' library. Please install it manually.")
            sys.exit(1)

    def two_element(self, soup):
         # ---------store title,image_link,
        a=soup.find_all('a',class_="ipc-metadata-list-item__list-content-item ipc-metadata-list-item__list-content-item--link")
        
        na=soup.find('span',class_="sc-afe43def-1 fDTGTb")
        name=na.text
        
        print(name)
        self.title.append(name)
        image=soup.find('img',class_="ipc-image").get('src')
        self.poster.append(image)
        # -------store director and writer name
        director1 = writer1 = ''
        for i in range(len(a)):
            if i == 0:
                director1 = a[i].text
            elif i == 1:
                writer1 = a[i].text

        self.director.append(director1)
        self.writer.append(writer1)  
        #------- store release date,also known as,film_location      
        b=soup.find_all('li')
        for j in b:   

            if j.find('a')==None:
                pass  
            elif j.find('a').text=='Release date':
                w1=j.find('a',class_="ipc-metadata-list-item__list-content-item ipc-metadata-list-item__list-content-item--link")
                self.release.append(w1.text)
            elif j.find('a').text=='Also known as':
                w2=j.find('span',class_="ipc-metadata-list-item__list-content-item")
                self.also_known_as.append(w2.text)
            elif j.find('a').text=='Filming locations':
                w3=j.find('a',class_="ipc-metadata-list-item__list-content-item ipc-metadata-list-item__list-content-item--link")
                self.film_location.append(w3.text)
        # -------store country of origin,language
        
        r=soup.find_all('li',class_="ipc-metadata-list__item")
        for i in r:
            if i.find('span')==None:
                pass
            elif i.find('span').text in ["Country of origin" , "Countries of origin"]:
                a1=i.find('a',class_="ipc-metadata-list-item__list-content-item ipc-metadata-list-item__list-content-item--link").text
                self.country_of_origin.append(a1)
            elif i.find('span').text=="Languages":
                a2=i.find('a',class_="ipc-metadata-list-item__list-content-item ipc-metadata-list-item__list-content-item--link").text
                self.language.append(a2)


        

    def three_elements(self, soup):
        a = soup.find_all('li', class_="ipc-inline-list__item sc-f1de8f0-1 dMosNF")
        scores = [i.find('span', class_='score').text for i in a]

        try:
            return scores[0], scores[1], scores[2]
        except IndexError:
            return '', '', ''
    
    

    def get_top_250_movies(self):
        count = 0 
        main_url = "https://www.imdb.com/chart/top/"
        response = requests.get(main_url, headers=self.headers)
        soup1 = BeautifulSoup(response.text, 'html.parser')
        #old class
        # movie_rows = soup1.find('ul', class_="ipc-metadata-list ipc-metadata-list--dividers-between sc-3a353071-0 wTPeg compact-list-view ipc-metadata-list--base")
        #new class
        movie_rows = soup1.find('ul', class_="ipc-metadata-list ipc-metadata-list--dividers-between sc-9d2f6de0-0 ckQYbL compact-list-view ipc-metadata-list--base")
    

    
       
        for row in movie_rows.find_all('li')[248:]:
            count=count+1
            
            
            link = "https://www.imdb.com" + row.find('a', class_="ipc-title-link-wrapper").get('href')
            print(f"Processing movie {count}: {link}")  # Example output
            self.movielink.append(link)
            response = requests.get(link, headers=self.headers)
            soup = BeautifulSoup(response.text, 'html.parser')

            user_score, critic_score, meta_score = self.three_elements(soup)
            self.user_viewers.append(user_score)
            self.critic_viewers.append(critic_score)
            self.metadata.append(meta_score)

            self.two_element(soup)

            movie_rating = soup.find('span', class_="sc-bde20123-1 iZlgcd").text
            self.rating.append(movie_rating)
            viewer = soup.find('div', class_="sc-bde20123-3 bjjENQ").text
            self.viewers.append(viewer)

            aa = soup.find_all('li', class_="ipc-metadata-list__item")
            for i in aa:
                al = i.find_all('span')
                for j in al:
                    if j.text == "Runtime":
                        p = i.find('div')
                        self.time.append(p.text)
            

    def generate_excel(self, filename):
        max_length = max(len(lst) for lst in [self.user_viewers, self.critic_viewers, self.metadata, self.rating,
                                              self.viewers, self.director, self.writer, self.release,
                                              self.country_of_origin, self.language, self.also_known_as,
                                              self.film_location, self.title, self.poster, self.time])
        for lst in [self.user_viewers, self.critic_viewers, self.metadata, self.rating,
                    self.viewers, self.release, self.country_of_origin, self.language,
                    self.also_known_as, self.film_location, self.time]:
            lst += ['None'] * (max_length - len(lst))

        data = {
            'User Viewers': self.user_viewers,
            'Critic Viewers': self.critic_viewers,
            'Metadata': self.metadata,
            'Rating': self.rating,
            'Viewers': self.viewers,
            'Director': self.director,
            'Writer': self.writer,
            'Release Date': self.release,
            'Country of Origin': self.country_of_origin,
            'Language': self.language,
            'Also Known As': self.also_known_as,
            'Film Location': self.film_location,
            'Title': self.title,
            'Runtime': self.time,
            'Poster': self.poster,
            'movie_link': self.movielink
        }

        df = pd.DataFrame(data)
        df.to_excel(filename, index=False)

